using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.PortalControls;
using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.CMSHelper;
using CMS.SettingsProvider;
using CMS.ExtendedControls;
using CMS.DataEngine;
using CMS.EmailEngine;
using CMS.SiteProvider;
using CMS.EventLog;
using CMS.URLRewritingEngine;
using CMS.Community;

public partial class CMSWebParts_Membership_LogonForm : CMSAbstractWebPart
{
    #region "Private properties"

    private string mDefaultTargetUrl = "";

    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets the value that indicates whether retrieving of forgotten password is enabled
    /// </summary>
    public bool AllowPasswordRetrieval
    {
        get
        {
            return ValidationHelper.GetBoolean(this.GetValue("AllowPasswordRetrieval"), true);
        }
        set
        {
            this.SetValue("AllowPasswordRetrieval", value);
            this.lnkPasswdRetrieval.Visible = value;
        }
    }


    /// <summary>
    /// Gets or sets the sender e-mail (from)
    /// </summary>
    public string SendEmailFrom
    {
        get
        {
            return DataHelper.GetNotEmpty(ValidationHelper.GetString(this.GetValue("SendEmailFrom"), "admin@domain.com"), "admin@domain.com");
        }
        set
        {
            this.SetValue("SendEmailFrom", value);
        }
    }


    /// <summary>
    /// Gets or sets the default target url (rediredction when the user is logged in)
    /// </summary>
    public string DefaultTargetUrl
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("DefaultTargetUrl"), mDefaultTargetUrl);
        }
        set
        {
            this.SetValue("DefaultTargetUrl", value);
            this.mDefaultTargetUrl = value;
        }
    }


    /// <summary>
    /// Gets or sets the SkinID of the logon form
    /// </summary>
    public override string SkinID
    {
        get
        {
            return base.SkinID;
        }
        set
        {
            base.SkinID = value;
            SetSkinID(value);
        }
    }


    /// <summary>
    /// Gets or sets the logon failure text
    /// </summary>
    public string FailureText
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("FailureText"), "");
        }
        set
        {
            if (value.ToString().Trim() != "")
            {
                this.SetValue("FailureText", value);
                this.Login1.FailureText = value;
            }
        }
    }


    /// <summary>
    /// Gets or sets the group name. Only members of this group can log on
    /// </summary>
    public string AllowedGroup
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("AllowedGroup"), "");
        }
        set
        {
            this.SetValue("AllowedGroup", value);        
        }
    }

    #endregion


    /// <summary>
    /// Content loaded event handler
    /// </summary>
    public override void OnContentLoaded()
    {
        base.OnContentLoaded();
        SetupControl();
    }


    /// <summary>
    /// Initializes the control properties
    /// </summary>
    protected void SetupControl()
    {
        if (this.StopProcessing)
        {
            this.rqValue.Visible = false;
            // Do not process
        }
        else
        {
            // Set failure text
            if (this.FailureText != "")
            {
                Login1.FailureText = ResHelper.LocalizeString(this.FailureText);
            }
            else
            {
                Login1.FailureText = ResHelper.GetString("Login_FailureText");
            }

            // Set strings
            lnkPasswdRetrieval.Text = ResHelper.GetString("LogonForm.lnkPasswordRetrieval");
            lblPasswdRetrieval.Text = ResHelper.GetString("LogonForm.lblPasswordRetrieval");
            btnPasswdRetrieval.Text = ResHelper.GetString("LogonForm.btnPasswordRetrieval");
            rqValue.ErrorMessage = ResHelper.GetString("LogonForm.rqValue");

            // Set logon strings
            LocalizedLabel lblItem = (LocalizedLabel)Login1.FindControl("lblUserName");
            if (lblItem != null)
            {
                lblItem.Text = "{$LogonForm.UserName$}";
            }
            lblItem = (LocalizedLabel)Login1.FindControl("lblPassword");
            if (lblItem != null)
            {
                lblItem.Text = "{$LogonForm.Password$}";
            }
            LocalizedCheckBox chkItem = (LocalizedCheckBox)Login1.FindControl("chkRememberMe");
            if (chkItem != null)
            {
                chkItem.Text = "{$LogonForm.RememberMe$}";
            }
            LocalizedButton btnItem = (LocalizedButton)Login1.FindControl("LoginButton");
            if (btnItem != null)
            {
                btnItem.Text = "{$LogonForm.LogOnButton$}";
            }

            this.ltlScript.Text = ScriptHelper.GetScript("function ShowPasswdRetrieve(id){var hdnDispl = document.getElementById('" + this.hdnPasswDisplayed.ClientID + "'); style=document.getElementById(id).style;style.display=(style.display == 'block')?'none':'block'; if(hdnDispl != null){ if(style.display == 'block'){hdnDispl.value='1';}else{hdnDispl.value='';}}}");
            this.lnkPasswdRetrieval.Attributes.Add("onclick", "ShowPasswdRetrieve('" + pnlPasswdRetrieval.ClientID + "'); return false;");

            this.lnkPasswdRetrieval.Visible = this.AllowPasswordRetrieval;

            Login1.LoggedIn += new EventHandler(Login1_LoggedIn);
            Login1.LoggingIn += new LoginCancelEventHandler(Login1_LoggingIn);

            btnPasswdRetrieval.Click += new EventHandler(btnPasswdRetrieval_Click);
            
            if (!RequestHelper.IsPostBack())
            {
                Login1.UserName = ValidationHelper.GetString(Request.QueryString["username"], "");
                // Set SkinID properties
                if (!this.StandAlone && (this.PageCycle < PageCycleEnum.Initialized)&&(ValidationHelper.GetString(this.Page.StyleSheetTheme,"") == ""))
                {
                    SetSkinID(this.SkinID);
                }
            }

            if (!this.AllowPasswordRetrieval)
            {
                pnlPasswdRetrieval.Visible = false;
            }
        }
    }


    /// <summary>
    /// OnLoad override (show hide password retrieval)
    /// </summary>
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        if (hdnPasswDisplayed.Value == "")
        {
            this.pnlPasswdRetrieval.Attributes.Add("style", "display:none;");
        }
        else
        {
            this.pnlPasswdRetrieval.Attributes.Add("style", "display:block;");
        }
    }


    /// <summary>
    /// Sets SkinId to all controls in logon form
    /// </summary>
    void SetSkinID(string skinId)
    {
        if (skinId != "")
        {
            Login1.SkinID = skinId;

            LocalizedLabel lbl = (LocalizedLabel)this.Login1.FindControl("lblUserName");
            if (lbl != null)
            {
                lbl.SkinID = skinId;
            }
            lbl = (LocalizedLabel)this.Login1.FindControl("lblPassword");
            if (lbl != null)
            {
                lbl.SkinID = skinId;
            }

            TextBox txt = (TextBox)this.Login1.FindControl("UserName");
            if (txt != null)
            {
                txt.SkinID = skinId;
            }
            txt = (TextBox)this.Login1.FindControl("Password");
            if (txt != null)
            {
                txt.SkinID = skinId;
            }

            LocalizedCheckBox chk = (LocalizedCheckBox)this.Login1.FindControl("chkRememberMe");
            if (chk != null)
            {
                chk.SkinID = skinId;
            }

            LocalizedButton btn = (LocalizedButton)this.Login1.FindControl("LoginButton");
            if (btn != null)
            {
                btn.SkinID = skinId;
            }
        }
    }


    /// <summary>
    /// Applies given stylesheet skin
    /// </summary>
    public override void ApplyStyleSheetSkin(Page page)
    {
        SetSkinID(this.SkinID);
        
        base.ApplyStyleSheetSkin(page);
    }


    /// <summary>
    /// Retrieve the user password
    /// </summary>
    void btnPasswdRetrieval_Click(object sender, EventArgs e)
    {
        string value = txtPasswordRetrieval.Text.Trim();
        
        if (value != "")
        {
            string userParam = txtPasswordRetrieval.Text.Trim().Replace("'", "''");

            DataSet ds = UserInfoProvider.GetUsers("UserName=N'" + userParam + "' OR Email=N'" + userParam + "'", null);

            // User exists, send email with password
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                string siteName = CMSContext.CurrentSiteName;
                // Get user info
                UserInfo user = new UserInfo(ds.Tables[0].Rows[0], null, null);

                // Quit if user do not have an email
                if (string.IsNullOrEmpty(user.Email))
                {
                    lblResult.Text = ResHelper.GetString("LogonForm.NoEmail");
                    this.pnlPasswdRetrieval.Attributes.Add("style", "display:block;");
                    lblResult.Visible = true;
                    return;
                }

                // Send password
                EmailTemplateInfo emailTemplate = EmailTemplateProvider.GetEmailTemplate("forgottenPassword", siteName);
                if (emailTemplate != null)
                {
                    EmailMessage message = new EmailMessage();
                    message.EmailFormat = EmailFormatEnum.Default;
                    message.From = this.SendEmailFrom;
                    message.Subject = ResHelper.GetString("LogonForm.ForgottenPasswordSubject");
                    message.Recipients = user.Email;
                    message.Body = emailTemplate.TemplateText;

                    // Get the password
                    string password = ValidationHelper.GetString(user.GetValue("UserPassword"), "");
                    switch (user.PasswordFormat.ToLower())
                    {
                        // SHA1 hash of the password, generate new
                        case "sha1":
                            password = UserInfoProvider.GenerateNewPassword();
                            break;
                    }

                    // Macros
                    string[,] macros = new string[3, 2];
                    macros[0, 0] = "LogonUrl";
                    macros[0, 1] = HttpContext.Current.Request.UrlReferrer.AbsoluteUri;
                    macros[1, 0] = "UserName";
                    macros[1, 1] = user.UserName;
                    macros[2, 0] = "Password";
                    macros[2, 1] = password;

                    // Create macro resolver
                    MacroResolver resolver = new MacroResolver();
                    resolver.SourceParameters = macros;

                    try
                    {
                        // Attach template metafiles to e-mail
                        MetaFileInfoProvider.ResolveMetaFileImages(message, emailTemplate.TemplateID, EmailObjectType.EMAILTEMPLATE, MetaFileInfoProvider.OBJECT_CATEGORY_TEMPLATE);
                        // Send email
                        EmailSender.SendEmailWithTemplateText(siteName, message, emailTemplate, resolver, true);

                        // Set new password after email was send successfully
                        UserInfoProvider.SetPassword(user.UserName, password);
                        lblResult.Text = ResHelper.GetString("LogonForm.PasswordSent") + " " + HTMLHelper.HTMLEncode(user.Email) + ".";
                    }
                    catch (Exception ex)
                    {
                        EventLogProvider eventLog = new EventLogProvider();
                        eventLog.LogEvent("Password retrieval", "LOGONFORM", ex);
                        lblResult.Text = "Failed to send the password: " +  ex.Message;
                    }
                }
            }
            // Wrong username or email, inform user
            else
            {
                lblResult.Text = ResHelper.GetString("LogonForm.NoUser");
                this.pnlPasswdRetrieval.Attributes.Add("style", "display:block;");
            }
            lblResult.Visible = true;
        }
    }


    /// <summary>
    /// Logged in handler
    /// </summary>
    void Login1_LoggedIn(object sender, EventArgs e)
    {
        // Ensure response cookie
        CookieHelper.EnsureResponseCookie(FormsAuthentication.FormsCookieName);

        // Set cookie expiration
        if (Login1.RememberMeSet)
        {
            CookieHelper.ChangeCookieExpiration(FormsAuthentication.FormsCookieName, DateTime.Now.AddYears(1), false);
        }
        else
        {
            // Extend the expiration of the authentication cookie if required
            if (!UserInfoProvider.UseSessionCookies && (HttpContext.Current != null) && (HttpContext.Current.Session != null))
            {
                CookieHelper.ChangeCookieExpiration(FormsAuthentication.FormsCookieName, DateTime.Now.AddMinutes(Session.Timeout), false);
            }
        }

        // Redirect user to the return url, or if is not defined redirect to the default target url
        if (ValidationHelper.GetString(Request.QueryString["ReturnURL"], "") != "")
        {
            UrlHelper.Redirect(ResolveUrl(ValidationHelper.GetString(Request.QueryString["ReturnURL"], "")));
        }
        else
        {
            if (this.DefaultTargetUrl != "")
            {
                UrlHelper.Redirect(ResolveUrl(this.DefaultTargetUrl));
            }
            else
            {
                UrlHelper.Redirect(URLRewriter.CurrentURL);
            }
        }
    }

    
    /// <summary>
    /// Ligging in handler
    /// </summary>
    void Login1_LoggingIn(object sender, LoginCancelEventArgs e)
    {
        // Ban IP addresses which are blocked for login
        if (!BannedIPInfoProvider.IsAllowed(CMSContext.CurrentSite.SiteID, BanControlEnum.Login))
        {
            e.Cancel = true;

            LocalizedLiteral failureLit = Login1.FindControl("FailureText") as LocalizedLiteral;
            if (failureLit != null)
            {
                failureLit.Visible = true;
                failureLit.Text = ResHelper.GetString("banip.ipisbannedlogin");
            }                        
        }


        if (CMSContext.CurrentSite != null) {
            UserInfo ui = UserInfoProvider.GetUserInfo(Login1.UserName);
            if (ui != null) {
                if (!ui.IsInSite(CMSContext.CurrentSiteName))
                {
                    e.Cancel = true;

                    LocalizedLiteral failureLit = Login1.FindControl("FailureText") as LocalizedLiteral;
                    if (failureLit != null)
                    {
                        failureLit.Visible = true;
                        failureLit.Text = DataHelper.GetNotEmpty(this.FailureText, ResHelper.GetString("login_failuretext"));
                    }  
                }
            }
        }


        if (!String.IsNullOrEmpty(this.AllowedGroup))
        {
            UserInfo ui = UserInfoProvider.GetUserInfo(Login1.UserName);
            GroupInfo gi = GroupInfoProvider.GetGroupInfo(this.AllowedGroup, CMSContext.CurrentSiteName);

            if ((ui != null) && (gi != null))
            {
                if (!GroupMemberInfoProvider.IsMemberOfGroup(ui.UserID, gi.GroupID))
                {
                    e.Cancel = true;

                    LocalizedLiteral failureLit = Login1.FindControl("FailureText") as LocalizedLiteral;
                    if (failureLit != null)
                    {
                        failureLit.Visible = true;
                        failureLit.Text = DataHelper.GetNotEmpty(this.FailureText, ResHelper.GetString("login_failuretext"));
                    }
                }
            }
        }

        if (((CheckBox)Login1.FindControl("chkRememberMe")).Checked)
        {
            Login1.RememberMeSet = true;
        }
        else
        {
            Login1.RememberMeSet = false;
        }
    }


    ///<summary>
    /// Overrides the generation of the SPAN tag with custom tag.
    ///</summary>
    protected HtmlTextWriterTag TagKey
    {
        get
        {
            if (CMSContext.CurrentSite != null)
            {
                if (SettingsKeyProvider.GetStringValue(CMSContext.CurrentSiteName + ".CMSControlElement").ToLower().Trim() == "div")
                {
                    return HtmlTextWriterTag.Div;
                }
                else
                {
                    return HtmlTextWriterTag.Span;
                }
            }
            return HtmlTextWriterTag.Span;
        }
    }
}
